@@ -7,9 +7,12 @@
 #' @docType data
 #' @keywords datasets
 #' @format A \code{reflector_spct} object with 226 rows and 2 variables
-#' @name green_leaf.spct
 #' @family Spectral data examples
 #' @references
 #' Aphalo, P. J. & Lehto, T. Effects of light quality on growth and N
 #' accumulation in birch seedlings Tree Physiology, 1997, 17, 125-132
-NULL
+#'
+#' @examples
+#' green_leaf.spct
+#'
+"green_leaf.spct"
